@@ -2,7 +2,7 @@
    This file is part of SecureSkat.
 
  Copyright (C) 2004, 2005, 2006, 2007,
-                                 2017  Heiko Stamer <HeikoStamer@gmx.net>
+               2017, 2018, 2019  Heiko Stamer <HeikoStamer@gmx.net>
 
    SecureSkat is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,340 +21,6 @@
 
 #include "SecureSkat_game.hh"
 
-// values for biding
-size_t reiz_wert[] =
-	{
-		0, 18, 20, 22, 23, 24, 27, 30, 33, 35, 36, 40, 44, 45, 46, 48, 50, 54, 
-		55, 59, 60, 63, 66, 70, 72, 77, 80, 81, 84, 88, 90, 96, 99, 100, 108, 
-		110, 117, 120, 121, 126, 130, 132, 135, 140, 143, 144, 150, 153, 154, 
-		156, 160, 162, 165, 168, 170, 176, 180, 187, 192, 198, 204, 216, 240
-	};
-
-// values of the cards (Augen)
-size_t pkt_wert[] =
-	{
-		2, 2, 2, 2, 
-		11, 10, 4, 3, 0, 0, 0, 11, 10, 4, 3, 0, 0, 0, 
-		11, 10, 4, 3, 0, 0, 0, 11, 10, 4, 3, 0, 0, 0
-	};
-
-// order of the cards in several games (99 is a marker for an invalid card)
-size_t card_order[][5][18] =
-	{
-		// Sc
-		{
-			{
-				0,  1,  2,  3, 25, 26, 27, 28, 29, 30, 31,  4,  5,  6,  7,  8,  9, 10
-			},
-			{
-				0,  1,  2,  3, 25, 26, 27, 28, 29, 30, 31, 11, 12, 13, 14, 15, 16, 17
-			},
-			{
-				0,  1,  2,  3, 25, 26, 27, 28, 29, 30, 31, 18, 19, 20, 21, 22, 23, 24, 
-			},
-			{
-				0,  1,  2,  3, 25, 26, 27, 28, 29, 30, 31, 99, 99, 99, 99, 99, 99, 99,
-			},
-			{
-				0,  1,  2,  3, 25, 26, 27, 28, 29, 30, 31, 99, 99, 99, 99, 99, 99, 99
-			}
-		},
-		// Ro
-		{
-			{
-				0,  1,  2,  3, 18, 19, 20, 21, 22, 23, 24,  4,  5,  6,  7,  8,  9, 10
-			},
-			{
-				0,  1,  2,  3, 18, 19, 20, 21, 22, 23, 24, 11, 12, 13, 14, 15, 16, 17
-			},
-			{
-				0,  1,  2,  3, 18, 19, 20, 21, 22, 23, 24, 99, 99, 99, 99, 99, 99, 99
-			},
-			{
-				0,  1,  2,  3, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31
-			},
-			{
-				0,  1,  2,  3, 18, 19, 20, 21, 22, 23, 24, 99, 99, 99, 99, 99, 99, 99
-			}
-		},
-		// Gr
-		{
-			{
-				0,  1,  2,  3, 11, 12, 13, 14, 15, 16, 17,  4,  5,  6,  7,  8,  9, 10
-			},
-			{
-				0,  1,  2,  3, 11, 12, 13, 14, 15, 16, 17, 99, 99, 99, 99, 99, 99, 99
-			},
-			{
-				0,  1,  2,  3, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24
-			},
-			{
-				0,  1,  2,  3, 11, 12, 13, 14, 15, 16, 17, 25, 26, 27, 28, 29, 30, 31
-			},
-			{
-				0,  1,  2,  3, 11, 12, 13, 14, 15, 16, 17, 99, 99, 99, 99, 99, 99, 99
-			}
-		},
-		// Ei
-		{
-			{
-				0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 99, 99, 99, 99, 99, 99, 99
-			},
-			{
-				0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14, 15, 16, 17
-			},
-			{
-				0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 18, 19, 20, 21, 22, 23, 24
-			},
-			{
-				0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 25, 26, 27, 28, 29, 30, 31
-			},
-			{
-				0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 99, 99, 99, 99, 99, 99, 99
-			}
-		},
-		// Nu
-		{
-			{
-				4,  6,  7,  0,  5,  8,  9, 10, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99
-			},
-			{
-				11, 13, 14,  1, 12, 15, 16, 17, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99
-			},
-			{
-				18, 20, 21,  2, 19, 22, 23, 24, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99
-			},
-			{
-				25, 27, 28,  3, 26, 29, 30, 31, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99
-			},
-			{
-				99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99
-			}
-		},
-		// Gd
-		{
-			{
-				0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 99, 99, 99, 99, 99, 99, 99
-			},
-			{
-				0,  1,  2,  3, 11, 12, 13, 14, 15, 16, 17, 99, 99, 99, 99, 99, 99, 99
-			},
-			{
-				0,  1,  2,  3, 18, 19, 20, 21, 22, 23, 24, 99, 99, 99, 99, 99, 99, 99
-			},
-			{
-				0,  1,  2,  3, 25, 26, 27, 28, 29, 30, 31, 99, 99, 99, 99, 99, 99, 99
-			},
-			{
-				0,  1,  2,  3, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99
-			}
-		}
-	};
-
-size_t skat_idx
-	(
-		size_t ft[5][18], size_t f, size_t t, size_t s
-	)
-{
-	assert (s < 18);
-	assert (f < 5);
-	for (size_t i = s; i < 18; i++)
-		if (t == ft[f][i])
-			return i;
-	return 99;
-}
-
-size_t skat_spiel2gwert
-	(
-		size_t spiel
-	)
-{
-	return (spiel % 100);
-}
-
-size_t skat_spiel2twert
-	(
-		size_t spiel
-	)
-{
-	if (skat_spiel2gwert(spiel) == 9)
-		return 0;
-	else if (skat_spiel2gwert(spiel) == 10)
-		return 1;
-	else if (skat_spiel2gwert(spiel) == 11)
-		return 2;
-	else if (skat_spiel2gwert(spiel) == 12)
-		return 3;
-	else if (skat_spiel2gwert(spiel) == 23)
-		return 4;
-	else if (skat_spiel2gwert(spiel) == 24)
-		return 5;
-	else
-		return 99;
-}
-
-void init_co
-	(
-		size_t spiel, size_t co[5][18]
-	)
-{
-	for (size_t i = 0; i < 5; i++)
-		for (size_t j = 0; j < 18; j++)
-			co[i][j] = card_order[skat_spiel2twert(spiel)][i][j];
-}
-
-size_t skat_spitzen
-	(
-		size_t spiel, SchindelhauerTMCG *tmcg,
-		const TMCG_OpenStack<VTMF_Card> &os
-	)
-{
-	size_t co[5][18], sz_cnt = 0;
-	init_co(spiel, co);
-	
-	// Null Spiel? Keine Spitzen!
-	if (skat_spiel2gwert(spiel) == 23)
-		return 0;
-	
-	// mit oder ohne Spitzen zaehlen
-	if (os.find(0))
-	{
-		for (size_t sz = 0; sz < 11; sz++)
-		{
-			if (os.find(co[4][sz]))
-					sz_cnt++;
-			else
-				break;
-		}
-	}
-	else
-	{
-		for (size_t sz = 0; sz < 11; sz++)
-		{
-			if (!os.find(co[4][sz]))
-				sz_cnt++;
-			else
-				break;
-		}
-	}
-	return sz_cnt;
-}
-
-bool skat_rulectl
-	(
-		size_t t, size_t tt, size_t spiel, const std::vector<size_t> &cv
-	)
-{
-	TMCG_OpenStack<VTMF_Card> os;
-	for (size_t j = 0; j < cv.size(); j++)
-		os.push(cv[j], VTMF_Card());
-	return skat_rulectl(t, tt, spiel, os);
-}
-
-bool skat_rulectl
-	(
-		size_t t, size_t tt, size_t spiel,
-		const TMCG_OpenStack<VTMF_Card> &os
-	)
-{
-	size_t co[5][18], to = 0;
-	init_co(spiel, co);
-	
-	if (skat_spiel2gwert(spiel) == 23)
-		to = 0;	// Null: kein Trumpf
-	else if (skat_spiel2gwert(spiel) == 24)
-		to = 4;	// Grand: 4 Truempfe
-	else
-		to = 11;	// Farbspiele: 11 Truempfe
-
-	if (skat_idx(co, 4, t, 0) != 99)
-	{
-		// Trumpf angespielt, aber nicht bedient
-		if (skat_idx(co, 4, tt, 0) == 99)
-		{
-			for (size_t i = 0; i < os.size(); i++)
-				if (skat_idx(co, 4, os[i].first, 0) < 99)
-						return false;
-			return true;
-		}
-		return true;
-	}
-	else
-	{
-		// Farbe i angespielt ...
-		for (size_t i = 0; i < 4; i++)
-		{
-			if (skat_idx(co, i, t, 0) != 99)
-			{
-				// ... aber nicht bedient
-				if (skat_idx(co, i, tt, to) == 99)
-				{
-					for (size_t j = 0; j < os.size(); j++)
-						if (skat_idx(co, i, os[j].first, to) < 99)
-							return false;
-					return true;
-				}
-				return true;
-			}
-		}
-	}
-	return false;
-}
-	
-int skat_bstich
-	(
-		const TMCG_OpenStack<VTMF_Card> &os, size_t spiel
-	)
-{
-	size_t co[5][18];
-	assert (os.size() == 3);
-	init_co(spiel, co);
-
-	if (skat_idx(co, 4, os[0].first, 0) != 99)
-	{
-		// Trumpf angespielt
-		if (skat_idx(co, 4, os[1].first, 0) < skat_idx(co, 4, os[0].first, 0))
-		{
-			if (skat_idx(co, 4, os[2].first, 0) < skat_idx(co, 4, os[1].first, 0))
-				return 2;
-			else
-				return 1;
-		}
-		else
-		{
-			if (skat_idx(co, 4, os[2].first, 0) < skat_idx(co, 4, os[0].first, 0))
-				return 2;
-			else
-				return 0;
-		}
-	}
-	else
-	{
-		// Farbe i angespielt
-		for (size_t i = 0; i < 4; i++)
-		{
-			if (skat_idx(co, i, os[0].first, 0) != 99)
-			{
-				if (skat_idx(co, i, os[1].first, 0) < skat_idx(co, i, os[0].first, 0))
-				{
-					if (skat_idx(co, i, os[2].first, 0) < skat_idx(co, i, os[1].first, 0))
-						return 2;
-					else
-						return 1;
-				}
-				else
-				{
-					if (skat_idx(co, i, os[2].first, 0) < skat_idx(co, i, os[0].first, 0))
-						return 2;
-					else
-						return 0;
-				}
-			}
-		}
-	}
-	return -1;
-}
-
 int skat_vkarte
 	(
 		size_t pkr_self, size_t pkr_who, SchindelhauerTMCG *tmcg,
@@ -363,20 +29,16 @@ int skat_vkarte
 	)
 {
 	assert(pkr_self != pkr_who);
-	
 #ifndef NDEBUG
 	start_clock();
 #endif
-	
-	int type = -1;
-	VTMF_Card c;
-	
 	try
 	{
 		if (((pkr_self == 0) && (pkr_who == 1)) || 
 			((pkr_self == 1) && (pkr_who == 2)) || 
 			((pkr_self == 2) && (pkr_who == 0)))
 		{
+			VTMF_Card c;
 			*left >> c;
 			if (!left->good())
 				throw -1;
@@ -385,24 +47,28 @@ int skat_vkarte
 			tmcg->TMCG_SelfCardSecret(c, vtmf);
 			if (!tmcg->TMCG_VerifyCardSecret(c, vtmf, *left, *left))
 				throw -1;
-			
 			if ((pkr_self == 0) && (pkr_who == 1))
 				tmcg->TMCG_ProveCardSecret(c, vtmf, *right, *right);
 			if ((pkr_self == 1) && (pkr_who == 2))
+			{
 				if (!tmcg->TMCG_VerifyCardSecret(c, vtmf, *right, *right))
 					throw -1;
+			}
 			if ((pkr_self == 2) && (pkr_who == 0))
+			{
 				if (!tmcg->TMCG_VerifyCardSecret(c, vtmf, *right, *right))
 					throw -1;
+			}
 			if ((pkr_self == 0) && (pkr_who == 1))
+			{
 				if (!tmcg->TMCG_VerifyCardSecret(c, vtmf, *right, *right))
 					throw -1;
+			}
 			if ((pkr_self == 1) && (pkr_who == 2))
 				tmcg->TMCG_ProveCardSecret(c, vtmf, *right, *right);
 			if ((pkr_self == 2) && (pkr_who == 0))
 				tmcg->TMCG_ProveCardSecret(c, vtmf, *right, *right);
-			
-			type = tmcg->TMCG_TypeOfCard(c, vtmf);
+			int type = tmcg->TMCG_TypeOfCard(c, vtmf);
 			if (rmv)
 				s.remove(c);
 			throw type;
@@ -411,6 +77,7 @@ int skat_vkarte
 			((pkr_self == 1) && (pkr_who == 0)) || 
 			((pkr_self == 2) && (pkr_who == 1)))
 		{
+			VTMF_Card c;
 			*right >> c;
 			if (!right->good())
 				throw -1;
@@ -419,24 +86,28 @@ int skat_vkarte
 			tmcg->TMCG_SelfCardSecret(c, vtmf);
 			if (!tmcg->TMCG_VerifyCardSecret(c, vtmf, *right, *right))
 				throw -1;
-			
 			if ((pkr_self == 0) && (pkr_who == 2))
 				tmcg->TMCG_ProveCardSecret(c, vtmf, *left, *left);
 			if ((pkr_self == 1) && (pkr_who == 0))
 				tmcg->TMCG_ProveCardSecret(c, vtmf, *left, *left);
 			if ((pkr_self == 2) && (pkr_who == 1))
+			{
 				if (!tmcg->TMCG_VerifyCardSecret(c, vtmf, *left, *left))
 					throw -1;
+			}
 			if ((pkr_self == 0) && (pkr_who == 2))
+			{
 				if (!tmcg->TMCG_VerifyCardSecret(c, vtmf, *left, *left))
 					throw -1;
+			}
 			if ((pkr_self == 1) && (pkr_who == 0))
+			{
 				if (!tmcg->TMCG_VerifyCardSecret(c, vtmf, *left, *left))
 					throw -1;
+			}
 			if ((pkr_self == 2) && (pkr_who == 1))
 				tmcg->TMCG_ProveCardSecret(c, vtmf, *left, *left);
-			
-			type = tmcg->TMCG_TypeOfCard(c, vtmf);
+			int type = tmcg->TMCG_TypeOfCard(c, vtmf);
 			if (rmv)
 				s.remove(c);
 			throw type;
@@ -462,7 +133,6 @@ void skat_okarte
 #ifndef NDEBUG
 	start_clock();
 #endif
-	
 	// use the non-interactiveness of the proof (only VTMF!)
 	std::stringstream proof;
 	tmcg->TMCG_ProveCardSecret(c, vtmf, proof, proof);
@@ -470,88 +140,12 @@ void skat_okarte
 	*right << proof.str() << std::flush;
 	*left << c << std::endl << std::flush;
 	*left << proof.str() << std::flush;
-	
 #ifndef NDEBUG
 	stop_clock();
 	std::cerr << elapsed_time() << std::flush;
 #endif
 }
 
-std::string skat_spiel2string
-	(
-		size_t spiel
-	)
-{
-	std::string wstr = "";
-	if (skat_spiel2gwert(spiel) == 9)
-		wstr += "Schellen (Sc)";
-	else if (skat_spiel2gwert(spiel) == 10)
-		wstr += "Rot (Ro)";
-	else if (skat_spiel2gwert(spiel) == 11)
-		wstr += "Gruen (Gr)";
-	else if (skat_spiel2gwert(spiel) == 12)
-		wstr += "Eicheln (Ei)";
-	else if (skat_spiel2gwert(spiel) == 23)
-		wstr += "Null (Nu)";
-	else if (skat_spiel2gwert(spiel) == 24)
-		wstr += "Grand (Gd)";
-	else
-		wstr += _("unknown");
-	if (spiel > 1000)
-	{
-		wstr += " Hand";
-		if (((spiel % 1000) > 100) && !(skat_spiel2gwert(spiel) == 23))
-			wstr += " Schneider";
-		if (((spiel % 1000) > 200) && !(skat_spiel2gwert(spiel) == 23))
-			wstr += " Schwarz";
-		if (((spiel % 1000) > 100) && !(skat_spiel2gwert(spiel) == 23))
-			wstr += _(" announced");
-		if ((spiel % 1000) > 300)
-			wstr += " Ouvert";
-	}
-	else
-	{
-		if ((spiel > 100) && !(skat_spiel2gwert(spiel) == 23))
-			wstr += " Schneider";
-		if ((spiel > 200) && !(skat_spiel2gwert(spiel) == 23))
-			wstr += " Schwarz";
-		if ((spiel > 100) && !(skat_spiel2gwert(spiel) == 23))
-			wstr += _(" announced");
-		if (spiel > 300)
-			wstr += " Ouvert";
-	}
-	return std::string(wstr);
-}
-
-int skat_wort2spiel
-	(
-		const std::string &wort
-	)
-{
-	if (wort == "")
-		return 0;
-	else if (wort == "Sc")
-		return 9;
-	else if (wort == "Ro")
-		return 10;
-	else if (wort == "Gr")
-		return 11;
-	else if (wort == "Nu")
-		return 23;
-	else if (wort == "Ei")
-		return 12;
-	else if (wort == "Gd")
-		return 24;
-	else if (wort == "Sn")
-		return 100;
-	else if (wort == "Sw")
-		return 200;
-	else if (wort == "Ov")
-		return 300;
-	else
-		return -1;
-}
-
 void skat_szeigen
 	(
 		SchindelhauerTMCG *tmcg, BarnettSmartVTMF_dlog *vtmf,
@@ -598,128 +192,6 @@ bool skat_ssehen
 	return true;
 }
 
-int skat_wort2type
-	(
-		const std::string &wort
-	)
-{
-	// Wenzel
-	if (wort == "EiU")
-		return 0;
-	else if (wort == "GrU")
-		return 1;
-	else if (wort == "RoU")
-		return 2;
-	else if (wort == "ScU")
-		return 3;
-	else
-	{
-		if (wort.length() < 3)
-			return -1;
-		std::string wert = wort.substr(2, wort.length() - 2);
-		int ww = -1;
-		// Figuren und Luschen
-		if (wert == "A")
-			ww = 4;
-		else if (wert == "10")
-			ww = 5;
-		else if (wert == "K")
-			ww = 6;
-		else if (wert == "O")
-			ww = 7;
-		else if (wert == "9")
-			ww = 8;
-		else if (wert == "8")
-			ww = 9;
-		else if (wert == "7")
-			ww = 10;
-		else
-			return -1;
-		if (wort.find("Ei", 0) == 0)
-			return ww;
-		else if (wort.find("Gr", 0) == 0)
-			return ww + 7;
-		else if (wort.find("Ro", 0) == 0)
-			return ww + 14;
-		else if (wort.find("Sc", 0) == 0)
-			return ww + 21;
-		else
-			return -1;
-	}
-}
-
-std::string skat_type2string
-	(
-		size_t type
-	)
-{
-	std::string wstr = "";
-	// Wenzel
-	if (type == 0)
-		wstr += "EiU ";
-	else if (type == 1)
-		wstr += "GrU ";
-	else if (type == 2)
-		wstr += "RoU ";
-	else if (type == 3)
-		wstr += "ScU ";
-	else
-	{
-		// Farben
-		int ww = type - 4;
-		if (ww < 7)
-			wstr += "Ei";
-		else if (ww < 14)
-			wstr += "Gr", ww -= 7;
-		else if (ww < 21)
-			wstr += "Ro", ww -= 14;
-		else if (ww < 28)
-			wstr += "Sc", ww -= 21;
-		// Figuren und Luschen
-		if (ww == 0)
-			wstr += "A ";
-		else if (ww == 1)
-			wstr += "10 ";
-		else if (ww == 2)
-			wstr += "K ";
-		else if (ww == 3)
-			wstr += "O ";
-		else if (ww == 4)
-			wstr += "9 ";
-		else if (ww == 5)
-			wstr += "8 ";
-		else if (ww == 6)
-			wstr += "7 ";
-	}
-	return std::string(wstr);
-}
-
-void skat_blatt
-	(
-		size_t p, const TMCG_OpenStack<VTMF_Card> &os
-	)
-{
-	std::vector<int> w;
-	for (size_t i = 0; i < os.size(); i++)
-		w.push_back(os[i].first);
-	if (p != 99)
-		std::sort(w.begin(), w.end());
-	if (p != 99)
-		std::cout << "><><>< ";
-	if (p == 0)
-		std::cout << "VH: ";
-	if (p == 1)
-		std::cout << "MH: ";
-	if (p == 2)
-		std::cout << "HH: ";
-	if (p == 10)
-		std::cout << _("revealed cards") << ": ";
-	for (std::vector<int>::const_iterator wi = w.begin(); wi != w.end(); wi++)
-		std::cout << skat_type2string(*wi);
-	if (p != 99)
-		std::cout << std::endl;
-}
-
 bool skat_sehen
 	(
 		size_t pkr_self, SchindelhauerTMCG *tmcg, BarnettSmartVTMF_dlog *vtmf,
@@ -781,7 +253,7 @@ bool skat_sehen
 
 bool skat_geben
 	(
-		SchindelhauerTMCG *tmcg, TMCG_Stack<VTMF_Card> &d_mix,
+		TMCG_Stack<VTMF_Card> &d_mix,
 		TMCG_Stack<VTMF_Card> &s0, TMCG_Stack<VTMF_Card> &s1,
 		TMCG_Stack<VTMF_Card> &s2, TMCG_Stack<VTMF_Card> &sk
 	)
@@ -964,7 +436,7 @@ bool game_helper_1
 			reiz_status += 100;
 			std::cout << "><><>< VH aka \"" << pkr.keys[vh].name << 
 				"\" " << _("gets the game at") << " " << 
-				reiz_wert[reiz_counter] << std::endl;
+				skat_reizwert[reiz_counter] << std::endl;
 			spiel_allein = vh;
 			if (pctl)
 				*out_ctl << nicks[spiel_allein] << " SPIELT" << std::endl << 
@@ -974,7 +446,7 @@ bool game_helper_1
 			reiz_status += 100;
 			std::cout << "><><>< MH aka \"" << pkr.keys[mh].name << 
 				"\" " << _("gets the game at") << " " << 
-				reiz_wert[reiz_counter] << std::endl;
+				skat_reizwert[reiz_counter] << std::endl;
 			spiel_allein = mh;
 			if (pctl)
 				*out_ctl << nicks[spiel_allein] << " SPIELT" << std::endl << 
@@ -984,7 +456,7 @@ bool game_helper_1
 			reiz_status += 100;
 			std::cout << "><><>< HH aka \"" << pkr.keys[hh].name << 
 				"\" " << _("gets the game at") << " " << 
-				reiz_wert[reiz_counter] << std::endl;
+				skat_reizwert[reiz_counter] << std::endl;
 			spiel_allein = hh;
 			if (pctl)
 				*out_ctl << nicks[spiel_allein] << " SPIELT" << std::endl << 
@@ -1002,13 +474,14 @@ int skat_game
 		TMCG_PublicKeyRing &pkr, const TMCG_SecretKey &sec,
 		iosecuresocketstream *right, iosecuresocketstream *left,
 		const std::vector<std::string> &nicks, int hpipe, bool pctl,
-		char *ireadbuf, int &ireaded,
+		char *ireadbuf, size_t &ireaded,
 		std::string main_channel, std::string main_channel_underscore
 	)
 {
 	if (!gcry_md_get_algo_dlen(GCRY_MD_RMD160))
 	{
-		std::cout << ">< " << _("ERROR") << ": " << _("gcry_md_get_algo_dlen() failed") << std::endl;
+		std::cout << ">< " << _("ERROR") << ": " <<
+			_("gcry_md_get_algo_dlen() failed") << std::endl;
 		return 1;
 	}
 
@@ -1022,7 +495,8 @@ int skat_game
 	for (size_t i = 0; pctl && (i < 3); i++)
 	{
 		std::ostringstream ost;
-		ost << nicks[pkr_self] << " INIT " << nicks[i] << " " << pkr.keys[i].name << std::endl;
+		ost << nicks[pkr_self] << " INIT " << nicks[i] << " " <<
+			pkr.keys[i].name << std::endl;
 		*out_ctl << ost.str() << std::flush;
 	}
 	
@@ -1052,7 +526,8 @@ int skat_game
 	}
 	if (!vtmf->CheckGroup())
 	{
-		std::cout << ">< " << _("VTMF ERROR") << ": " << _("function CheckGroup() failed") << std::endl;
+		std::cout << ">< " << _("VTMF ERROR") << ": " <<
+			_("function CheckGroup() failed") << std::endl;
 		delete vtmf;
 		if (pctl)
 			delete out_ctl;
@@ -1245,7 +720,8 @@ int skat_game
 #ifndef NDEBUG
 			start_clock();
 #endif
-			if (!skat_mischen(pkr_self, tmcg, vtmf, d2, ss, d_mix[0], d_mix[1], d_mix[2], right, left))
+			if (!skat_mischen(pkr_self, tmcg, vtmf, d2, ss, d_mix[0], d_mix[1],
+				d_mix[2], right, left))
 			{
 				std::cout << ">< " << _("shuffling error") << ": " << _("bad stack format") << std::endl;
 				delete vsshe;
@@ -1263,7 +739,8 @@ int skat_game
 #ifndef NDEBUG
 			start_clock();
 #endif
-			if (!skat_mischen_beweis(pkr_self, tmcg, vtmf, vsshe, d2, ss, d_mix[0], d_mix[1], d_mix[2], right, left))
+			if (!skat_mischen_beweis(pkr_self, tmcg, vtmf, vsshe, d2, ss,
+				d_mix[0], d_mix[1], d_mix[2], right, left))
 			{
 				std::cout << ">< " << _("shuffling error") << ": " << _("wrong ZK proof") << std::endl;
 				delete vsshe;
@@ -1287,7 +764,8 @@ int skat_game
 			std::string osttmp = game_stream.str();
 
 			char *game_digest = new char[dlen];
-			gcry_md_hash_buffer(GCRY_MD_RMD160, game_digest, osttmp.c_str(), osttmp.length());
+			gcry_md_hash_buffer(GCRY_MD_RMD160, game_digest, osttmp.c_str(),
+				osttmp.length());
 			char *hex_game_digest =	new char[2 * dlen + 1];
 			for (size_t i = 0; i < dlen; i++)
 				snprintf(hex_game_digest + (2 * i), 3, "%02x", (unsigned char)game_digest[i]);
@@ -1305,7 +783,7 @@ int skat_game
 			start_clock();
 #endif
 			TMCG_Stack<VTMF_Card> s[3], sk;
-			if (!skat_geben(tmcg, d_end, s[0], s[1], s[2], sk))
+			if (!skat_geben(d_end, s[0], s[1], s[2], sk))
 			{
 				std::cout << ">< " << _("dealing error") << ": " << _("not enough cards") << std::endl;
 				delete [] hex_game_digest;
@@ -1318,7 +796,8 @@ int skat_game
 			}
 			std::cout << "." << std::flush;
 			TMCG_OpenStack<VTMF_Card> os, os_ov, os_sp, os_st, os_pkt[3], os_rc[3];
-			if (!skat_sehen(pkr_self, tmcg, vtmf, os, s[0], s[1], s[2], right, left))
+			if (!skat_sehen(pkr_self, tmcg, vtmf, os, s[0], s[1], s[2],
+				right, left))
 			{
 				std::cout << ">< " << _("dealing error") << ": " << _("wrong ZK proof") << std::endl;
 				delete [] hex_game_digest;
@@ -1373,13 +852,17 @@ int skat_game
 					return 5;
 				}
 				
-				if (!game_helper_1(reiz_status, spiel_allein, vh, mh, hh, reiz_counter, pkr_self, nicks, pkr, pctl, out_ctl))
+				if (!game_helper_1(reiz_status, spiel_allein, vh, mh, hh,
+					reiz_counter, pkr_self, nicks, pkr, pctl, out_ctl))
+				{
 					break;
+				}
 				
 				// select(2) -- initialize file descriptors
 				FD_ZERO(&rfds);
-				MFD_SET(ipipe, &rfds);
-				if (pctl)
+				if (ipipe < FD_SETSIZE)
+					MFD_SET(ipipe, &rfds);
+				if (pctl && (ctl_i < FD_SETSIZE))
 					MFD_SET(ctl_i, &rfds);
 				
 				// select(2)
@@ -1407,16 +890,25 @@ int skat_game
 					num = read(ctl_i, ireadbuf + ireaded, 65536 - ireaded);
 //std::cerr << "ctl_i got " << num << std::endl;
 				}
-				ireaded += num;
+				if (num <= 0)
+				{
+					std::cerr << _("read error in skat_game() encountered") <<
+						" [errno=" << errno << "]" << std::endl;
+					break;
+				}
+				else
+					ireaded += num;
 				
 				if (ireaded > 0)
 				{
 					bool got_break = false;
-					std::vector<int> pos_delim;
-					int cnt_delim = 0, cnt_pos = 0, pos = 0;
-					for (int i = 0; i < ireaded; i++)
+					std::vector<size_t> pos_delim;
+					size_t cnt_delim = 0, cnt_pos = 0, pos = 0;
+					for (size_t i = 0; i < ireaded; i++)
+					{
 						if (ireadbuf[i] == '\n')
 							cnt_delim++, pos_delim.push_back(i);
+					}
 					while (cnt_delim >= 1)
 					{
 						char xtmp[65536];
@@ -1425,7 +917,7 @@ int skat_game
 						--cnt_delim, cnt_pos = pos_delim[pos] + 1, pos++;
 						std::string cmd = xtmp;
 						// do operation
-						// ---------------------------------------------------------------
+						// ------------
 				
 				if (!left->good() || !right->good())
 				{
@@ -1544,7 +1036,7 @@ int skat_game
 									break;
 							}
 							std::cout << "><><>< " << _("player") << " \"" << pkr.keys[who_biding].name << "\" " << _("passes at") << " " << 
-								reiz_wert[reiz_counter] << std::endl;
+								skat_reizwert[reiz_counter] << std::endl;
 							if (pctl)
 								*out_ctl << nicks[who_biding] << " PASSE" << std::endl << std::flush;
 						}
@@ -1565,7 +1057,7 @@ int skat_game
 							switch (reiz_status)
 							{
 								case 0: // MH reizt (zu Beginn)
-									if (irw != reiz_wert[++reiz_counter])
+									if (irw != skat_reizwert[++reiz_counter])
 									{
 										delete [] hex_game_digest;
 										delete vsshe;
@@ -1578,7 +1070,7 @@ int skat_game
 									reiz_status = 1;
 									break;
 								case 1: // VH sagt ja (nach Reizen MH)
-									if (irw != reiz_wert[reiz_counter])
+									if (irw != skat_reizwert[reiz_counter])
 									{
 										delete [] hex_game_digest;
 										delete vsshe;
@@ -1591,7 +1083,7 @@ int skat_game
 									reiz_status = 2;
 									break;
 								case 2: // MH reizt (weiter)
-									if (irw != reiz_wert[++reiz_counter])
+									if (irw != skat_reizwert[++reiz_counter])
 									{
 										delete [] hex_game_digest;
 										delete vsshe;
@@ -1605,7 +1097,7 @@ int skat_game
 									break;
 								case 3: // HH reizt (nach sofort Passen MH, oder Reizen VH)
 								case 5: // HH reizt (nach sofort Passen MH, oder Reizen VH)
-									if (irw != reiz_wert[++reiz_counter])
+									if (irw != skat_reizwert[++reiz_counter])
 									{
 										delete [] hex_game_digest;
 										delete vsshe;
@@ -1618,7 +1110,7 @@ int skat_game
 									reiz_status = 4;
 									break;
 								case 4: // VH sagt ja (nach Passen MH, Reizen HH)
-									if (irw != reiz_wert[reiz_counter])
+									if (irw != skat_reizwert[reiz_counter])
 									{
 										delete [] hex_game_digest;
 										delete vsshe;
@@ -1631,7 +1123,7 @@ int skat_game
 									reiz_status = 5;
 									break;
 								case 6: // HH reizt (nach Passen MH)
-									if (irw != reiz_wert[++reiz_counter])
+									if (irw != skat_reizwert[++reiz_counter])
 									{
 										delete [] hex_game_digest;
 										delete vsshe;
@@ -1644,7 +1136,7 @@ int skat_game
 									reiz_status = 7;
 									break;
 								case 7: // VH sagt ja (nach Passen MH, Reizen HH)
-									if (irw != reiz_wert[reiz_counter])
+									if (irw != skat_reizwert[reiz_counter])
 									{
 										delete [] hex_game_digest;
 										delete vsshe;
@@ -1657,7 +1149,7 @@ int skat_game
 									reiz_status = 6;
 									break;
 								case 8: // HH reizt (nach Passen VH)
-									if (irw != reiz_wert[++reiz_counter])
+									if (irw != skat_reizwert[++reiz_counter])
 									{
 										delete [] hex_game_digest;
 										delete vsshe;
@@ -1670,7 +1162,7 @@ int skat_game
 									reiz_status = 9;
 									break;
 								case 9: // MH sagt ja (nach Passen VH)
-									if (irw != reiz_wert[reiz_counter])
+									if (irw != skat_reizwert[reiz_counter])
 									{
 										delete [] hex_game_digest;
 										delete vsshe;
@@ -1683,7 +1175,7 @@ int skat_game
 									reiz_status = 8;
 									break;
 								case 10: // VH reizt (nach sofort Passen MH, sofort Passen HH)
-									if (irw != reiz_wert[++reiz_counter])
+									if (irw != skat_reizwert[++reiz_counter])
 									{
 										delete [] hex_game_digest;
 										delete vsshe;
@@ -1705,11 +1197,11 @@ int skat_game
 									return 23;
 							}
 							std::cout << "><><>< " << _("player") << " \"" << pkr.keys[who_biding].name << "\" " << _("bids") << " " << 
-								reiz_wert[reiz_counter] << std::endl;
+								skat_reizwert[reiz_counter] << std::endl;
 							if (pctl)
 							{
 								std::ostringstream ost;
-								ost << nicks[who_biding] << " REIZE " << reiz_wert[reiz_counter] << std::endl;
+								ost << nicks[who_biding] << " REIZE " << skat_reizwert[reiz_counter] << std::endl;
 								*out_ctl << ost.str() << std::flush;
 							}
 						}
@@ -2090,7 +1582,7 @@ int skat_game
 						if (pass_ok)
 						{
 							std::cout << "><><>< " << _("player") << " \"" << pkr.keys[pkr_self].name << "\" " << _("passes at") << " " << 
-								reiz_wert[reiz_counter] << std::endl;
+								skat_reizwert[reiz_counter] << std::endl;
 							*out_pipe << "PRIVMSG " << main_channel_underscore << nr << 
 								" :PASSE " << hex_game_digest << std::endl << std::flush;
 							if (pctl)
@@ -2166,13 +1658,13 @@ int skat_game
 						if (bid_ok)
 						{
 							std::cout << "><><>< " << _("player") << " \"" << pkr.keys[pkr_self].name << "\" " << _("bids") << " " << 
-								reiz_wert[reiz_counter] << std::endl;
-							*out_pipe << "PRIVMSG " << main_channel_underscore << nr << " :REIZE " << reiz_wert[reiz_counter] << " " << 
+								skat_reizwert[reiz_counter] << std::endl;
+							*out_pipe << "PRIVMSG " << main_channel_underscore << nr << " :REIZE " << skat_reizwert[reiz_counter] << " " << 
 								hex_game_digest << std::endl << std::flush;
 							if (pctl)
 							{
 								std::ostringstream ost;
-								ost << nicks[pkr_self] << " REIZE " << reiz_wert[reiz_counter] << std::endl;
+								ost << nicks[pkr_self] << " REIZE " << skat_reizwert[reiz_counter] << std::endl;
 								*out_ctl << ost.str() << std::flush;
 							}
 						}
@@ -2263,7 +1755,8 @@ int skat_game
 										int tt1 = skat_wort2type(cc1), tt2 = skat_wort2type(cc2);
 										if ((tt1 != -1) && (tt2 != -1))
 										{
-											if (os.find(tt1) && os.find(tt2))
+											if ((os.find(tt1) && os.find(tt2)) &&
+												(tt1 != tt2))
 											{
 												sk.clear();
 												os.move(tt1, sk), os.move(tt2, sk);
@@ -2532,8 +2025,10 @@ int skat_game
 				// Augen der Gegenpartei zaehlen
 				size_t pkt_allein = 0, pkt_gegner = 0; 
 				for (size_t i = 0; i < 3; i++)
+				{
 					for (size_t j = 0; (i != spiel_allein) && (j < os_pkt[i].size()); j++)
-						pkt_gegner += pkt_wert[os_pkt[i][j].first];
+						pkt_gegner += skat_pktwert[os_pkt[i][j].first];
+				}
 				pkt_allein = 120 - pkt_gegner;
 				std::cout << "><><>< " << _("card points") << " " <<
 					_("playing party") << " (" << pkr.keys[spiel_allein].name << "): " << 
@@ -2562,7 +2057,9 @@ int skat_game
 					{
 						if (!skat_rulectl(os_st[j].first,
 							os_rc[i][j].first, spiel_status, gps))
-								rules_ok[i] = false;
+						{
+							rules_ok[i] = false;
+						}
 						gps.remove(os_rc[i][j].first);
 					}
 				}
@@ -2600,7 +2097,7 @@ int skat_game
 				else
 				{
 					size_t spitzen = 
-						skat_spitzen(spiel_status, tmcg, os_rc[spiel_allein]);
+						skat_spitzen(spiel_status, os_rc[spiel_allein]);
 					size_t gs = spiel_status, gstufen = 1;
 					
 					// Hand (2)
@@ -2656,12 +2153,12 @@ int skat_game
 					
 					spiel_wert = (spitzen + gstufen) * skat_spiel2gwert(spiel_status);
 				}
-				if (reiz_wert[reiz_counter] > (size_t)spiel_wert)
+				if (skat_reizwert[reiz_counter] > (size_t)spiel_wert)
 				{
 					std::cout << "><>< " << _("playing party") << " " << 
 						_("has bet to much") << ". " << _("Loosing the game!") << std::endl;
 					spiel_wert = 0;
-					while ((size_t)spiel_wert < reiz_wert[reiz_counter])
+					while ((size_t)spiel_wert < skat_reizwert[reiz_counter])
 						spiel_wert += skat_spiel2gwert(spiel_status);
 					spiel_wert = -spiel_wert;
 					spiel_gewonnen = false;
@@ -2688,7 +2185,7 @@ int skat_game
 					}
 				}
 				std::cout << "><><>< " << _("game points") << ": " << spiel_wert << ", " << _("biding value") << ": " <<
-					reiz_wert[reiz_counter] << ", " << _("game outcome") << ": " <<
+					skat_reizwert[reiz_counter] << ", " << _("game outcome") << ": " <<
 					(spiel_gewonnen ? _("win") : _("loose")) << std::endl;
 				if (pctl)
 				{
@@ -2703,11 +2200,13 @@ int skat_game
 				std::ostringstream einzel_protokoll;
 				einzel_protokoll << nicks[spiel_allein] << "~" << spiel_wert <<
 					"~" << spiel_status << "~" << pkt_allein << "~" <<
-					reiz_wert[reiz_counter] << "~" << hex_game_digest << "~";
+					skat_reizwert[reiz_counter] << "~" << hex_game_digest << "~";
 				spiel_protokoll << einzel_protokoll.str() << "#";
 				if (pctl)
+				{
 					*out_ctl << nicks[spiel_allein] << " PROTO " <<
 						einzel_protokoll.str() << std::endl << std::flush;
+				}
 			}
 			else
 			{	
@@ -2863,9 +2362,10 @@ int skat_game
 	delete vsshe;
 	delete vtmf;
 	if (pctl)
+	{
 		*out_ctl << nicks[pkr_self] << " DONE" << std::endl << std::flush;
-	if (pctl)
 		delete out_ctl;
+	}
 	delete out_pipe;
 	return 0;
 }
