@@ -2,7 +2,7 @@
    This file is part of SecureSkat.
 
  Copyright (C) 2002, 2003, 2004, 2006, 2007, 2009,
-               2017, 2018 Heiko Stamer <HeikoStamer@gmx.net>
+               2017, 2018, 2019  Heiko Stamer <HeikoStamer@gmx.net>
 
    SecureSkat is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,8 +29,9 @@ int BindEmptyPort
 	while (current_port < (start_port + max_ports))
 	{
 		int sockfd;
-		long socket_option = 1;
-		struct sockaddr_in sin = { 0 };
+		int socket_option = 1;
+		struct sockaddr_in sin;
+		memset(&sin, 0, sizeof(sin));
 		sin.sin_port = htons(current_port);
 		sin.sin_family = AF_INET;
 		sin.sin_addr.s_addr = htonl(INADDR_ANY);
@@ -75,8 +76,9 @@ int ListenToPort
 	(int port)
 {
 	int sockfd;
-	long socket_option = 1;
-	struct sockaddr_in sin = { 0 };
+	int socket_option = 1;
+	struct sockaddr_in sin;
+	memset(&sin, 0, sizeof(sin));
 	sin.sin_port = htons(port);
 	sin.sin_family = AF_INET;
 	sin.sin_addr.s_addr = htonl(INADDR_ANY);
@@ -124,8 +126,8 @@ int CloseHandle
 int ConnectToHost
 	(const char *host, uint16_t port)
 {
-	struct addrinfo hints = { 0 }, *res, *rp;
-	hints.ai_family = AF_INET;
+	struct addrinfo hints = { 0, 0, 0, 0, 0, 0, 0, 0 }, *res, *rp;
+	hints.ai_family = AF_UNSPEC; //AF_INET; FIXME: resolving IPv4-only does not work
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_flags = AI_NUMERICSERV | AI_ADDRCONFIG;
 	std::stringstream ports;
@@ -163,6 +165,7 @@ int ConnectToHost
 		}
 	}
 	freeaddrinfo(res);
+	perror("SecureSkat_misc::ConnectToHost (connect)");					
 	return -2;
 }
 
