@@ -1,7 +1,7 @@
 /*******************************************************************************
    This file is part of SecureSkat.
 
- Copyright (C) 2007, 2017, 2018  Heiko Stamer <HeikoStamer@gmx.net>
+ Copyright (C) 2007, 2017, 2018, 2019  Heiko Stamer <HeikoStamer@gmx.net>
 
    SecureSkat is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -24,6 +24,9 @@ int ctl_pid = 0; // PID of the running control program
 
 RETSIGTYPE sig_handler_skat_quit(int sig)
 {
+	// two lines of dummy code follow to avoid compiler warnings
+	if (sig == 0)
+		sig = 0;
 #ifndef NDEBUG
 	std::cerr << "sig_handler_skat_quit: got signal " << sig << std::endl;
 #endif
@@ -41,7 +44,6 @@ extern TMCG_SecretKey sec;
 extern TMCG_PublicKey pub;
 extern std::map<std::string, TMCG_PublicKey> nick_key;
 extern std::map<std::string, std::string> nick_players;
-extern std::string X;
 extern std::string game_ctl;
 extern char **game_env;
 
@@ -58,7 +60,7 @@ int skat_connect
 	std::string tmp = "";
 	
 	// authenticate connection: receive the nonce
-	unsigned long int nonce_A = 0, nonce_B = 0;
+	unsigned long int nonce_A = 0;
 	*neighbor >> nonce_A;
 	neighbor->ignore(1, '\n');
 	if (neighbor->good())
@@ -68,7 +70,7 @@ int skat_connect
 		// send the signature
 		*neighbor << sec.sign(ost.str()) << std::endl << std::flush;
 		// create a fresh nonce
-		nonce_B = tmcg_mpz_srandom_ui();
+		unsigned long int nonce_B = tmcg_mpz_srandom_ui();
 		// send challenge
 		*neighbor << nonce_B << std::endl << std::flush;
 		// receive the signature
@@ -122,7 +124,7 @@ int skat_accept
 	(opipestream *out_pipe, int ipipe, const std::string &nr, int r,
 	int pkr_self, int pkr_idx, iosecuresocketstream *&secure, int &handle,
 	const std::vector<std::string> &vnicks, TMCG_PublicKeyRing &pkr,
-	int gp_handle, bool neu, char *ireadbuf, int &ireaded)
+	int gp_handle, bool neu, char *ireadbuf, size_t &ireaded)
 {
 	fd_set rfds;	// set of read descriptors
 	int mfds = 0;	// highest-numbered descriptor
@@ -130,14 +132,18 @@ int skat_accept
 	{
 		// select(2) -- initialize file descriptors
 		FD_ZERO(&rfds);
-		MFD_SET(gp_handle, &rfds);
-		MFD_SET(ipipe, &rfds);
+		if (gp_handle < FD_SETSIZE)
+			MFD_SET(gp_handle, &rfds);
+		if (ipipe < FD_SETSIZE)
+			MFD_SET(ipipe, &rfds);
 		// select(2)
 		int ret = select(mfds + 1, &rfds, NULL, NULL, NULL);
 		// error occured
 		if (ret < 0)
+		{
 			if (errno != EINTR)
 				perror("skat_accept (select)");
+		}
 		
 		// connection request
 		if ((ret > 0) && FD_ISSET(gp_handle, &rfds))
@@ -154,7 +160,7 @@ int skat_accept
 			{
 				iosocketstream *neighbor = new iosocketstream(handle);
 				// create a fresh nonce
-				unsigned long int nonce_A = tmcg_mpz_srandom_ui(), nonce_B = 0;
+				unsigned long int nonce_A = tmcg_mpz_srandom_ui();
 				std::ostringstream ost, ost2;
 				std::string tmp;
 				*neighbor << nonce_A << std::endl << std::flush;
@@ -171,6 +177,7 @@ int skat_accept
 				}
 				else
 				{
+					unsigned long int nonce_B = 0;
 					// receive the nonce and reply
 					*neighbor >> nonce_B;
 					neighbor->ignore(1, '\n');
@@ -220,14 +227,23 @@ int skat_accept
 		if ((ret > 0) && FD_ISSET(ipipe, &rfds))
 		{
 			ssize_t num = read(ipipe, ireadbuf + ireaded, 65536 - ireaded);
-			ireaded += num;
+			if (num <= 0)
+			{
+				std::cerr << _("read error in skat_accept() encountered") <<
+					" [errno=" << errno << "]" << std::endl;
+				break;
+			}
+			else
+				ireaded += num;
 			if (ireaded > 0)
 			{
-				std::vector<int> pos_delim;
-				int cnt_delim = 0, cnt_pos = 0, pos = 0;
-				for (int i = 0; i < ireaded; i++)
+				std::vector<size_t> pos_delim;
+				size_t cnt_delim = 0, cnt_pos = 0, pos = 0;
+				for (size_t i = 0; i < ireaded; i++)
+				{
 					if (ireadbuf[i] == '\n')
 						cnt_delim++, pos_delim.push_back(i);
+				}
 				while (cnt_delim >= 1)
 				{
 					char tmp[65536];
@@ -312,7 +328,7 @@ int skat_child
 	std::list<std::string> gp_nick;
 	std::map<std::string, std::string> gp_name;
 	char *ipipe_readbuf = new char[65536];
-	int ipipe_readed = 0;
+	size_t ipipe_readed = 0;
 	gp_nick.push_back(pub.keyid(5));
 	gp_name[pub.keyid(5)] = pub.name;
 	
@@ -438,7 +454,7 @@ int skat_child
 	std::vector<std::string> vnicks;
 	size_t pkr_i = 0, pkr_self = 0;
 	gp_nick.sort();
-	for (std::list<std::string>::const_iterator pi = gp_nick.begin(); pi != gp_nick.end(); pi++)
+	for (std::list<std::string>::const_iterator pi = gp_nick.begin(); pi != gp_nick.end(); ++pi)
 	{
 		vnicks.push_back(*pi);
 		if (*pi == pub.keyid(5))
@@ -616,9 +632,11 @@ int skat_child
 	if (neu)
 	{
 		// set topic
-		*out_pipe << "TOPIC " << MAIN_CHANNEL_UNDERSCORE << nr << " :" << PACKAGE_STRING << std::endl << std::flush;
+		*out_pipe << "TOPIC " << MAIN_CHANNEL_UNDERSCORE << nr <<
+			" :" << PACKAGE_STRING << std::endl << std::flush;
 	}
-	int exit_code = skat_game(nr, r, pkr_self, neu, opipe, ipipe, ctl_o, ctl_i, gp_tmcg, pkr, sec, right_neighbor, left_neighbor, vnicks, hpipe, pctl,
+	int exit_code = skat_game(nr, r, pkr_self, neu, opipe, ipipe, ctl_o, ctl_i,
+		gp_tmcg, pkr, sec, right_neighbor, left_neighbor, vnicks, hpipe, pctl,
 		ipipe_readbuf, ipipe_readed, MAIN_CHANNEL, MAIN_CHANNEL_UNDERSCORE);
 	
 	// stop gui or ai (control program)
