@@ -1,7 +1,7 @@
 /*******************************************************************************
    This file is part of SecureSkat.
 
- Copyright (C) 2007, 2009, 2017  Heiko Stamer <HeikoStamer@gmx.net>
+ Copyright (C) 2007, 2009, 2017, 2019  Heiko Stamer <HeikoStamer@gmx.net>
 
    SecureSkat is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,6 +22,9 @@
 
 RETSIGTYPE sig_handler_ballot_quit(int sig)
 {
+	// two lines of dummy code follow to avoid compiler warnings
+	if (sig == 0)
+		sig = 0;
 #ifndef NDEBUG
 	std::cerr << "signal_handler_ballot_quit: got signal " << sig << std::endl;
 #endif
@@ -32,7 +35,6 @@ extern TMCG_SecretKey sec;
 extern TMCG_PublicKey pub;
 extern std::map<std::string, TMCG_PublicKey> nick_key;
 extern std::map<std::string, std::string> nick_players;
-extern std::string X, XX, XXX;
 
 int ballot_child
 	(const std::string &nr, int b, bool neu, int ipipe, int opipe,
@@ -67,7 +69,10 @@ int ballot_child
 	gp_nick.push_back(pub.keyid(5));
 	gp_name[pub.keyid(5)] = pub.name;
 	if (neu)
-		*out_pipe << "PRIVMSG " << MAIN_CHANNEL << " :" << nr << "|1~" << -b << "!" << std::endl << std::flush;
+	{
+		*out_pipe << "PRIVMSG " << MAIN_CHANNEL << " :" << nr << "|1~" <<
+			-b << "!" << std::endl << std::flush;
+	}
 	
 	// wait for voters
 	while (1)
@@ -84,7 +89,8 @@ int ballot_child
 		}
 		if (neu && (cmd.find("!ANNOUNCE", 0) == 0))
 		{
-			*out_pipe << "PRIVMSG " << MAIN_CHANNEL << " :" << nr << "|" << gp_nick.size() << "~" << -b << "!" << std::endl << std::flush;
+			*out_pipe << "PRIVMSG " << MAIN_CHANNEL << " :" << nr << "|" <<
+				gp_nick.size() << "~" << -b << "!" << std::endl << std::flush;
 		}
 		if (neu && (cmd.find("JOIN ", 0) == 0))
 		{
@@ -95,9 +101,11 @@ int ballot_child
 				{
 					if (gp_nick.size() < TMCG_MAX_PLAYERS)
 					{
-						gp_nick.push_back(nick), gp_name[nick] = nick_key[nick].name;
-						*out_pipe << "PRIVMSG " << MAIN_CHANNEL << " :" << nr << "|" << 
-							gp_nick.size() << "~" << b << "!" << std::endl << std::flush;
+						gp_nick.push_back(nick);
+						gp_name[nick] = nick_key[nick].name;
+						*out_pipe << "PRIVMSG " << MAIN_CHANNEL << " :" <<
+							nr << "|" << gp_nick.size() << "~" << b << "!" <<
+							std::endl << std::flush;
 					}
 					else
 						*out_pipe << "KICK " << MAIN_CHANNEL_UNDERSCORE << nr << " " << 
@@ -182,7 +190,7 @@ int ballot_child
 	std::vector<std::string> vnicks;
 	size_t pkr_i = 0, pkr_self = 0;
 	gp_nick.sort();
-	for (std::list<std::string>::const_iterator pi = gp_nick.begin(); pi != gp_nick.end(); pi++, pkr_i++)
+	for (std::list<std::string>::const_iterator pi = gp_nick.begin(); pi != gp_nick.end(); ++pi, pkr_i++)
 	{
 		vnicks.push_back(*pi);
 		if (*pi == pub.keyid(5))
@@ -308,15 +316,17 @@ int ballot_child
 	int mfds = 0;				// highest-numbered descriptor
 	struct timeval tv;			// timeout structure
 	char *ireadbuf = new char[65536];
-	int ireaded = 0;
+	size_t ireaded = 0;
 	size_t pkr_idx = 0;
 	std::map<std::string, iosecuresocketstream*> ios_in, ios_out;
 	while (pkr_idx < gp_nick.size())
 	{
 		// select(2) -- initialize file descriptors
 		FD_ZERO(&rfds);
-		MFD_SET(gp_handle, &rfds);
-		MFD_SET(ipipe, &rfds);
+		if (gp_handle < FD_SETSIZE)
+			MFD_SET(gp_handle, &rfds);
+		if (ipipe < FD_SETSIZE)
+			MFD_SET(ipipe, &rfds);
 		
 		// select(2) -- initialize timeout
 		tv.tv_sec = 1L;	// seconds
@@ -435,14 +445,23 @@ int ballot_child
 		{
 			// pipe request
 			ssize_t num = read(ipipe, ireadbuf + ireaded, 65536 - ireaded);
-			ireaded += num;
+			if (num <= 0)
+			{
+				std::cerr << _("read error in ballot_child() encountered") <<
+					" [errno=" << errno << "]" << std::endl;
+				break;
+			}
+			else
+				ireaded += num;
 			if (ireaded > 0)
 			{
-				std::vector<int> pos_delim;
-				int cnt_delim = 0, cnt_pos = 0, pos = 0;
-				for (int i = 0; i < ireaded; i++)
+				std::vector<size_t> pos_delim;
+				size_t cnt_delim = 0, cnt_pos = 0, pos = 0;
+				for (size_t i = 0; i < ireaded; i++)
+				{
 					if (ireadbuf[i] == '\n')
 						cnt_delim++, pos_delim.push_back(i);
+				}
 				while (cnt_delim >= 1)
 				{
 					char tmp[65536];
